import os
import pytest


from imagesoup import ImageSoup, ImageResult
from imagesoup.utils import Blacklist


def test_creating_soup():
    soup = ImageSoup()
    assert isinstance(soup, ImageSoup)


def test_search_query_only_returns_100_images():
    soup = ImageSoup()
    images = soup.search('python')
    assert len(images) == 100


def test_search_n_images_set_by_user():
    N_IMAGES = 20
    soup = ImageSoup()
    images = soup.search('python', n_images=N_IMAGES)
    assert len(images) == N_IMAGES



def test_search_with_image_size_parameter():
    soup = ImageSoup()
    images = soup.search('python', image_size='large')
    im = images[0]
    width, height = im.size
    assert width > 500 and height > 500


def test_search_image_exact_size():
    soup = ImageSoup()
    size = (400, 400)
    images = soup.search('python', image_size=size)
    im = images[0]
    assert im.size == size


def test_search_image_aspect_ratio():
    soup = ImageSoup()
    images = soup.search('python', aspect_ratio='tall')
    im = images[0]
    assert im.height > im.width


def test_search_image_returns_fewer_results_than_n_images(capsys):
    soup = ImageSoup()
    images = soup.search('python', n_images=2000)
    out, err = capsys.readouterr()
    assert out.startswith('Search query "python" returned only')


def test_imageresult_url():
    soup = ImageSoup()
    images = soup.search('python site:python.org')
    im = images[0]
    assert im.URL == 'https://www.python.org/static/opengraph-icon-200x200.png'


def test_imageresult_show_image():
    soup = ImageSoup()
    images = soup.search('python')
    try:
        images[0].show()
    except:
        pytest.fail('Cant show image')


def test_imageresult_resize():
    soup = ImageSoup()
    images = soup.search('python')
    im = images[0]
    new_size = (400, 400)
    new_image = im.resize(new_size)
    assert new_image.size == new_size


def test_get_image_main_color():
    soup = ImageSoup()
    images = soup.search('white')
    im = images[0]
    main_color = im.main_color(reduce_size=True)
    assert len(main_color) == 1
    assert main_color[0][0] == 'white'


def test_imageresult_tofile():
    soup = ImageSoup()
    images = soup.search('pyhon site:python.org')
    im = images[0]
    im.to_file()
    STANDARD_NAME = 'image'
    assert os.path.isfile(STANDARD_NAME + '.png') is True
    os.remove(STANDARD_NAME + '.png')

    USER_INPUT_NAME = 'pythonlogo.png'
    im.to_file(USER_INPUT_NAME)
    assert os.path.isfile(USER_INPUT_NAME) is True
    os.remove(USER_INPUT_NAME)


def test_imageresult_verify_valid_file():
    soup = ImageSoup()
    images = soup.search('python site:python.org')
    im = images[0]
    assert im.verify() is True


def test_imageresult_verify_invalid_file():
    URL = 'https://httpstat.us/200'
    im = ImageResult(URL)
    assert im.verify() is False


def test_blacklist():
    bl = Blacklist()
    assert isinstance(bl, Blacklist)
    assert os.path.isfile(bl.filename) is True
    os.remove(bl.filename)


def test_blacklist_add():
    bl = Blacklist()
    bl.add('http://www.python.org')
    assert len(bl.domains) == 1
    assert bl.domains == ['python.org']


def test_blacklist_delete():
    bl = Blacklist()
    bl.delete('python.org')
    assert bl.domains == []


def test_blacklist_reset():
    bl = Blacklist()
    bl.add('http://www.python.org')
    bl.reset()
    assert bl.domains == []

def test_blacklist_query_string():
    bl = Blacklist()
    bl.add('http://www.python.org')
    bl.add('https://github.com/')
    query = '-site:python.org -site:github.com'
    assert bl.query_string() == query
    os.remove(bl.filename)



from collections import Counter

import webcolors


CSS3_COLORS = webcolors.CSS3_NAMES_TO_HEX.keys()


def color_analysis(image):
    if image.mode != 'RGB':
        image = image.convert('RGB')

    # initialize a Counter starting from zero
    color_counter = Counter({color: 0 for color in Counter(CSS3_COLORS)})
    for pixel_count, RGB in image.getcolors(image.width * image.height):
        color_name = get_color_name(RGB)
        color_counter[color_name] += pixel_count

    # Calculate percent for each color
    for color in color_counter:
        color_counter[color] = color_counter[color] / (image.width * image.height)

    return color_counter


def closest_color(requested_color):
    min_colors = {}
    for key, name in webcolors.css3_hex_to_names.items():
        r_c, g_c, b_c = webcolors.hex_to_rgb(key)
        rd = (r_c - requested_color[0]) ** 2
        gd = (g_c - requested_color[1]) ** 2
        bd = (b_c - requested_color[2]) ** 2
        min_colors[(rd + gd + bd)] = name
    return min_colors[min(min_colors.keys())]


def get_color_name(requested_color):
    try:
        color_name = webcolors.rgb_to_name(requested_color)
    except ValueError:
        color_name = closest_color(requested_color)
    return color_name



FILE_TYPE_OPTIONS = {}

USAGE_RIGHT_OPTIONS = {}

ASPECT_RATIO_OPTIONS = {'tall': 't', 'square': 's', 'wide': 'w',
                        'panoramic': 'xw'}

IMAGE_SIZE_OPTIONS = {'any': '', 'icon': 'i', 'medium': 'm', 'large': 'l',
                      'exactly': 'ex', '400x300+': 'qsvga', '640x480+': 'vga',
                      '800x600+': 'svga', '1024x768+': 'xga', '2mp+': '2mp',
                      '4mp+': '4mp', '6mp+': '6mp', '8mp+': '8mp',
                      '10mp+': '10mp', '12mp+': '12mp', '15mp+': '15mp',
                      '20mp+': '20mp', '40mp+': '40mp', '70mp+': '70mp'}


def aspect_ratio_paramenter(option):
    if not option:
        return None

    ASPECT_RATIO_PARAM = 'iar'
    return ASPECT_RATIO_PARAM + ':' + ASPECT_RATIO_OPTIONS[option]


def image_size_parameter(option):
    if not option:
        return None

    IMAGE_SIZE_PARAM = 'isz'
    if isinstance(option, (tuple, list)):
        width, height = option
        return IMAGE_SIZE_PARAM + ':{},iszw:{},iszh:{}'.format(IMAGE_SIZE_OPTIONS['exactly'], width, height)
    else:
        return IMAGE_SIZE_PARAM + ':' + IMAGE_SIZE_OPTIONS[option]


def image_aspect_parameters(aspect_ratio, image_size):
    if any([aspect_ratio, image_size]) is False:
        return None
    else:
        IMAGE_RELATED = 'tbs='
        options = ','.join([i for i in [aspect_ratio, image_size] if i is not None])
        return '{}{}'.format(IMAGE_RELATED, options)


def query_builder(query, image_size=None, aspect_ratio=None, page_number=0):
    if query is None:
        raise ValueError('query must have a value.')

    SEARCH_TYPE = 'tbm'
    IMAGES = 'isch'
    SEARCH_TYPE_PARAM = '='.join([SEARCH_TYPE, IMAGES])
    BASE_URL = 'https://www.google.com/search?' + SEARCH_TYPE_PARAM

    # Add page number
    PAGE_NUMBER = 'ijn'
    page_number_param = '='.join([PAGE_NUMBER, str(page_number)])
    URL = '&'.join([BASE_URL, page_number_param])

    # Add query value
    QUERY_TYPE = 'q'
    query_param = '='.join([QUERY_TYPE, str(query)])
    URL = '&'.join([URL, query_param])

    # Add image aspects parameters
    iar = aspect_ratio_paramenter(aspect_ratio)
    isz = image_size_parameter(image_size)
    image_aspect_param = image_aspect_parameters(iar, isz)
    if image_aspect_param is not None:
        URL = '&'.join([URL, image_aspect_param])

    return URL
